package org.dynmap.factions.listener;

import com.massivecraft.factions.entity.Faction;
import com.massivecraft.factions.event.EventFactionsChunksChange;
import com.massivecraft.factions.event.EventFactionsCreate;
import com.massivecraft.factions.event.EventFactionsDisband;
import com.massivecraft.factions.event.EventFactionsMembershipChange;
import com.massivecraft.factions.event.EventFactionsNameChange;
import com.massivecraft.factions.event.EventFactionsWarpAdd;
import com.massivecraft.factions.event.EventFactionsWarpRemove;
import org.bukkit.Bukkit;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.dynmap.factions.DynmapFactionsPlugin;
import org.dynmap.factions.runnable.FactionsUpdate;
import org.jetbrains.annotations.NotNull;

public class DynmapFactionsListener implements Listener {

    private final DynmapFactionsPlugin dynmapFactionsPlugin;

    public DynmapFactionsListener(DynmapFactionsPlugin dynmapFactionsPlugin) {
        this.dynmapFactionsPlugin = dynmapFactionsPlugin;
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFPlayerJoin(@NotNull EventFactionsMembershipChange event) {
        if (event.isCancelled()) {
            return;
        }
        if (dynmapFactionsPlugin.isPlayerSets()) {
            Faction f = event.getNewFaction();
            dynmapFactionsPlugin.requestUpdatePlayerSet(f.getId());
        }
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionCreate(@NotNull EventFactionsCreate event) {
        if (event.isCancelled()) {
            return;
        }
        if (dynmapFactionsPlugin.isPlayerSets()) {
            dynmapFactionsPlugin.requestUpdatePlayerSet(event.getFactionId());
        }
        requestUpdateFactions();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionDisband(@NotNull EventFactionsDisband event) {
        if (event.isCancelled()) {
            return;
        }
        if (dynmapFactionsPlugin.isPlayerSets()) {
            Faction f = event.getFaction();
            dynmapFactionsPlugin.requestUpdatePlayerSet(f.getId());
        }
        requestUpdateFactions();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionRename(@NotNull EventFactionsNameChange event) {
        if (event.isCancelled()) {
            return;
        }
        requestUpdateFactions();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionRename(@NotNull EventFactionsWarpAdd event) {
        if (event.isCancelled()) {
            return;
        }
        requestUpdateFactions();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionRename(@NotNull EventFactionsWarpRemove event) {
        if (event.isCancelled()) {
            return;
        }
        requestUpdateFactions();
    }

    @EventHandler(priority = EventPriority.MONITOR)
    public void onFactionRename(@NotNull EventFactionsChunksChange event) {
        if (event.isCancelled()) {
            return;
        }
        requestUpdateFactions();
    }

    private void requestUpdateFactions() {
        if (DynmapFactionsPlugin.getInstance().getPending() == null) {
            FactionsUpdate factionsUpdate = new FactionsUpdate(DynmapFactionsPlugin.getInstance(),
                    DynmapFactionsPlugin.getInstance().getConfig());
            factionsUpdate.runonce = true;
            DynmapFactionsPlugin.getInstance().setPending(factionsUpdate);
            Bukkit.getServer().getScheduler().scheduleSyncDelayedTask(DynmapFactionsPlugin.getInstance(), factionsUpdate, 20);
        }
    }

}
